@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.14 2004/06/15 12:57:55 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.15 2004/07/18 23:25:25 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -51,7 +51,6 @@ static pid_t fork_a_child(int unix_fd, int inet_fd);
 static int create_unix_domain_socket(void);
 static int create_inet_domain_socket(void);
 static void myexit(int code);
-static int check_pid_file(char *path);
 
 static RETSIGTYPE exit_handler(int sig);
 static RETSIGTYPE reap_handler(int sig);
@@ -208,7 +207,7 @@ int main(int argc, char **argv)
 
 static void usage(void)
 {
-	fprintf(stderr, "pgpool version %s,\n",	PGPOOLVERSION);
+	fprintf(stderr, "pgpool version %s(%s),\n",	VERSION, PGPOOLVERSION);
 	fprintf(stderr, "  a generic connection pool/replication/load balance server for PostgreSQL\n\n");
 	fprintf(stderr, "usage: pgpool [-f config_file][-n][-d][-h][stop]\n");
 	fprintf(stderr, "  config_file default path: %s/%s\n",DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
@@ -286,16 +285,6 @@ static void stop_me(void)
 	}
 }
 
-/*
-* check the existence of pid file. returns 0 if exists non 0 otherwise.
-*/
-static int check_pid_file(char *path)
-{
-	struct stat buf;
-
-	return stat(path, &buf);
-}
-
 /*
 * read the pid file
 */
