@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_auth.c,v 1.12 2004/07/07 12:26:14 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_auth.c,v 1.13 2004/07/19 01:28:44 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -47,7 +47,7 @@ static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reaut
 int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 {
 	int status;
-	char kind, kind1;
+	char kind;
 	int pid, pid1;
 	int key, key1;
 	int protoMajor;
@@ -209,9 +209,6 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	{
 		if (protoMajor == PROTO_MAJOR_V3)
 		{
-			char *p;
-			int len;
-
 			/* process parameter status */
 			while (kind == 'S')
 			{
@@ -279,26 +276,32 @@ int pool_do_reauth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 
 	protoMajor = MAJOR(cp);
 
-	/* trust? */
-	if (MASTER(cp)->auth_kind == 0)
-		status = 0;
-
-	/* clear text password? */
-	else if (MASTER(cp)->auth_kind == 3)
+	switch(MASTER(cp)->auth_kind)
 	{
-		status = do_clear_text_password(MASTER(cp), frontend, 1, protoMajor);
-	}
+		case 0:
+			/* trust */
+			status = 0;
+			break;
 
-	/* crypt password? */
-	else if (MASTER(cp)->auth_kind == 4)
-	{
-		status = do_crypt(MASTER(cp), frontend, 1, protoMajor);
-	}
+		case 3:
+			/* clear text password */
+			status = do_clear_text_password(MASTER(cp), frontend, 1, protoMajor);
+			break;
+			
+		case 4:
+			/* crypt password */
+			status = do_crypt(MASTER(cp), frontend, 1, protoMajor);
+			break;
 
-	/* md5 password? */
-	else if (MASTER(cp)->auth_kind == 5)
-	{
-		status = do_md5(MASTER(cp), frontend, 1, protoMajor);
+		case 5:
+			/* md5 password */
+			status = do_md5(MASTER(cp), frontend, 1, protoMajor);
+			break;
+
+		default:
+			pool_error("pool_do_reauth: unknown authentication request code %d", 
+					   MASTER(cp)->auth_kind);
+			return -1;
 	}
 
 	if (status == 0)
