@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.25 2004/07/16 11:02:35 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.27 2004/07/19 01:23:58 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -33,6 +33,8 @@
 
 #include <stdlib.h>
 #include <unistd.h>
+#include <string.h>
+#include <netinet/in.h>
 
 #include "pool.h"
 
@@ -152,10 +154,10 @@ POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 
 		}
 
-		if (!REPLICATION && MASTER(backend)->len == 0 && frontend->len == 0 ||
-			REPLICATION && MASTER(backend)->len == 0 &&
+		if ((!REPLICATION && MASTER(backend)->len == 0 && frontend->len == 0) ||
+			(REPLICATION && MASTER(backend)->len == 0 &&
 			SECONDARY(backend)->len == 0
-			&& frontend->len == 0)
+			 && frontend->len == 0))
 		{
 
 			struct timeval timeout;
@@ -497,8 +499,6 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 static POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend, 
 								 POOL_CONNECTION_POOL *backend, int send_ready)
 {
-	int len;
-
 	pool_flush(frontend);
 
 	if (send_ready)
@@ -678,13 +678,12 @@ static POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
 							POOL_CONNECTION_POOL *backend,
 							short num_fields)
 {
-	static char nullmap[8192];
-	static char nullmap1[8192];
+	static char nullmap[8192], nullmap1[8192];
 	int nbytes;
 	int i;
 	unsigned char mask;
 	int size, size1;
-	char *buf, *buf1;
+	char *buf;
 	char msgbuf[1024];
 
 	pool_write(frontend, "D", 1);
@@ -703,16 +702,16 @@ static POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
 	{
 		if (pool_read(SECONDARY(backend), nullmap1, nbytes) < 0)
 			return POOL_END;
-	}
 
-	if (memcmp(nullmap, nullmap1, nbytes))
-	{
-		/* XXX: NULLMAP maybe different among
-		   backends. If we were a paranoid, we have to treat
-		   this as a fatal error. However in the real world
-		   we'd better to adapt this situation. Just throw a
-		   log... */
-		pool_log("AsciiRow: NULLMAP differ between master and secondary");
+		if (memcmp(nullmap, nullmap1, nbytes))
+		{
+			/* XXX: NULLMAP maybe different among
+			   backends. If we were a paranoid, we have to treat
+			   this as a fatal error. However in the real world
+			   we'd better to adapt this situation. Just throw a
+			   log... */
+			pool_log("AsciiRow: NULLMAP differ between master and secondary");
+		}
 	}
 
 	mask = 0;
@@ -747,6 +746,8 @@ static POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
 			size1 = ntohl(size1) - 4;
 		}
 
+		buf = NULL;
+
 		if (mask & nullmap[i/8])
 		{
 			/* forward to frontend */
@@ -756,42 +757,23 @@ static POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
 			/* read and send actual data only when size > 0 */
 			if (size > 0)
 			{
-				buf = malloc(size);
+				buf = pool_read2(MASTER(backend), size);
 				if (buf == NULL)
-				{
-					pool_error("AsciiRow: out of memory");
 					return POOL_END;
-				}
-				/* actual data */
-				if (pool_read(MASTER(backend), buf, size) < 0)
-				{
-					free(buf);
-					return POOL_END;
-				}
 			}
 		}
 
 		if (REPLICATION && size1 > 0 && (mask & nullmap1[i/8]))
 		{
-			buf1 = malloc(size1);
-			if (buf1 == NULL)
-			{
-				pool_error("AsciiRow: out of memory");
+			/* read and discard secondary data */
+			if (pool_read2(SECONDARY(backend), size) == NULL)
 				return POOL_END;
-			}
-			if (pool_read(SECONDARY(backend), buf1, size1) < 0)
-			{
-				free(buf1);
-				return POOL_END;
-			}
-			free(buf1);
 		}
 
-		if ((mask & nullmap[i/8]) &&  size > 0)
+		if (buf)
 		{
 			pool_write(frontend, buf, size);
 			snprintf(msgbuf, Min(sizeof(msgbuf), size+1), "%s", buf);
-			free(buf);
 			pool_debug("AsciiRow: len: %d data: %s", size, msgbuf);
 		}
 
@@ -805,11 +787,11 @@ static POOL_STATUS BinaryRow(POOL_CONNECTION *frontend,
 							 POOL_CONNECTION_POOL *backend,
 							 short num_fields)
 {
-	static char nullmap[8192];
+	static char nullmap[8192], nullmap1[8192];
 	int nbytes;
 	int i;
 	unsigned char mask;
-	int size;
+	int size, size1;
 	char *buf;
 
 	pool_write(frontend, "B", 1);
@@ -821,12 +803,23 @@ static POOL_STATUS BinaryRow(POOL_CONNECTION *frontend,
 
 	/* NULL map */
 	pool_read(MASTER(backend), nullmap, nbytes);
-	if (REPLICATION)
-		pool_read(SECONDARY(backend), nullmap, nbytes);
-
 	if (pool_write(frontend, nullmap, nbytes) < 0)
-	{
 		return POOL_END;
+
+	if (REPLICATION)
+	{
+		if (pool_read(SECONDARY(backend), nullmap1, nbytes) < 0)
+			return POOL_END;
+
+		if (memcmp(nullmap, nullmap1, nbytes))
+		{
+			/* XXX: NULLMAP maybe different among
+			   backends. If we were a paranoid, we have to treat
+			   this as a fatal error. However in the real world
+			   we'd better to adapt this situation. Just throw a
+			   log... */
+			pool_log("BinaryRow: NULLMAP differ between master and secondary");
+		}
 	}
 
 	mask = 0;
@@ -840,32 +833,56 @@ static POOL_STATUS BinaryRow(POOL_CONNECTION *frontend,
 		if (mask & nullmap[i/8])
 		{
 			/* field size */
-			pool_read(MASTER(backend), &size, sizeof(int));
-			if (REPLICATION)
-				pool_read(SECONDARY(backend), &size, sizeof(int));
-			pool_write(frontend, &size, sizeof(int));
-			size = ntohl(size);
-			buf = malloc(size);
-			if (buf == NULL)
-			{
-				pool_error("BinaryRow: out of memory");
+			if (pool_read(MASTER(backend), &size, sizeof(int)) < 0)
 				return POOL_END;
-			}
-			/* actual data */
-			pool_read(MASTER(backend), buf, size);
-			if (REPLICATION)
-				pool_read(SECONDARY(backend), buf, size);
-			pool_write(frontend, buf, size);
-			free(buf);
 		}
+
+		if (REPLICATION && (mask & nullmap1[i/8]))
+		{
+			/* XXX: field size maybe different among
+			   backends. If we were a paranoid, we have to treat
+			   this as a fatal error. However in the real world
+			   we'd better to adapt this situation. Just throw a
+			   log... */
+
+			if (pool_read(SECONDARY(backend), &size1, sizeof(int)) < 0)
+				return POOL_END;
+
+			if (size != size1)
+				pool_log("BinaryRow: %d th field size does not match between master(%d) and secondary(%d)",
+						 i, ntohl(size), ntohl(size1));
+			size1 = ntohl(size1) - 4;
+		}
+
+		buf = NULL;
+
+		if (mask & nullmap[i/8])
+		{
+			/* forward to frontend */
+			pool_write(frontend, &size, sizeof(int));
+			size = ntohl(size) - 4;
+
+			/* read and send actual data only when size > 0 */
+			if (size > 0)
+			{
+				buf = pool_read2(MASTER(backend), size);
+				if (buf == NULL)
+					return POOL_END;
+			}
+		}
+
+		if (REPLICATION && size1 > 0 && (mask & nullmap1[i/8]))
+		{
+			/* read and discard secondary data */
+			if (pool_read2(SECONDARY(backend), size) == NULL)
+				return POOL_END;
+		}
+
+		if (buf)
+			pool_write(frontend, buf, size);
+
 		mask >>= 1;
 	}
-
-	if (pool_flush(frontend) < 0)
-	{
-		return POOL_END;
-	}
-
 	return POOL_CONTINUE;
 }
 
@@ -1202,21 +1219,12 @@ static POOL_STATUS FunctionCall(POOL_CONNECTION *frontend,
 
 		len = ntohl(len);
 
-		arg = malloc(len);
-		if (arg == NULL)
-		{
-			pool_error("FuncationCall: out of memory");
-			return POOL_ERROR;
-		}
-
 		/* argument value itself */
-		if (pool_read(frontend, arg, len) < 0)
+		if ((arg = pool_read2(frontend, len)) == NULL)
 			return POOL_ERROR;
 		pool_write(MASTER(backend), arg, len);
 		if (REPLICATION)
 			pool_write(SECONDARY(backend), arg, len);
-
-		free(arg);
 	}
 
 	if (pool_flush(MASTER(backend)))
@@ -1258,22 +1266,14 @@ static POOL_STATUS FunctionResultResponse(POOL_CONNECTION *frontend,
 
 		len = ntohl(len);
 
-		result = malloc(len);
-		if (result == NULL)
-		{
-			pool_error("FuncationResultResponse: out of memory");
-			return POOL_ERROR;
-		}
-
 		/* result value itself */
-		if (pool_read(MASTER(backend), result, len) < 0)
+		if ((result = pool_read2(MASTER(backend), len)) == NULL)
 			return POOL_ERROR;
 		if (REPLICATION)
 			if (pool_read(SECONDARY(backend), result, len) < 0)
 				return POOL_ERROR;
 
 		pool_write(frontend, result, len);
-		free(result);
 	}
 
 	/* unused ('0') */
@@ -1888,7 +1888,6 @@ static int reset_backend(POOL_CONNECTION_POOL *backend, int qcnt)
 
 	char *query;
 	int qn = sizeof(queries)/sizeof(char *);
-	int len;
 
 	if (qcnt >= qn)
 		return 2;
@@ -1987,7 +1986,6 @@ void pool_send_error_message(POOL_CONNECTION *frontend, int protoMajor,
 	{
 		char data[MAXDATA];
 		char msgbuf[MAXMSGBUF];
-		char *p;
 		int len;
 		int thislen;
 		int sendlen;
